@@ -24,7 +24,7 @@ import matplotlib
 matplotlib.use('Agg')  # Need this to write images from the GSA servers.  Order matters:
 import matplotlib.pyplot as plt  # First import matplotlib, then use Agg, then import plt
 
-from ml4h.models.legacy_models import make_multimodal_multitask_model
+from ml4h.models.legacy_models import legacy_multimodal_multitask_model
 from ml4h.TensorMap import TensorMap, Interpretation, decompress_data
 from ml4h.tensor_generators import TensorGenerator, test_train_valid_tensor_generators
 from ml4h.tensor_generators import BATCH_INPUT_INDEX, BATCH_OUTPUT_INDEX, BATCH_PATHS_INDEX
@@ -321,68 +321,23 @@ def sample_from_language_model(
     model, test_data, max_samples=16, heat=0.7,
 ):
     burn_in = np.zeros((1,) + language_input.shape, dtype=np.float32)
-    index_2_token = {v: k for k, v in language_output.channel_map.items()}
+    index_2_token = {v: str(k) for k, v in language_output.channel_map.items()}
     for i in range(min(max_samples, test_data[language_input.input_name()].shape[0])):  # iterate over the batch
         burn_in[0] = test_data[language_input.input_name()][i]
-        sentence = ''.join([index_2_token[index] for index in burn_in[0]])
-        logging.info(f' Batch    sentence start:{sentence}                   ------ {i}')
+        sentence = ''.join([index_2_token[index] for index in burn_in[0] if index in index_2_token])
+        logging.info(f'Input:{sentence}              ------ {i}')
         for j in range(max_samples):
             burn_in = np.zeros((1,) + language_input.shape, dtype=np.float32)
             for k, c in enumerate(sentence[j:]):
-                burn_in[0, k] = language_output.channel_map[c]
+                if c in language_output.channel_map:
+                    burn_in[0, k] = language_output.channel_map[c]
             cur_test = {language_input.input_name(): burn_in}
             prediction = model.predict(cur_test)
+            if isinstance(prediction, list):  # handle multitask
+                prediction = prediction[0]
             next_token = index_2_token[_sample_with_heat(prediction[0, -1, :], heat)]
             sentence += next_token
-        logging.info(f'Model completed sentence:{sentence}')
-
-
-def sample_from_char_embed_model(tensor_maps_in: List[TensorMap], char_model: Model, test_batch: Dict[str, np.ndarray], test_paths: List[str]) -> None:
-    for tm in tensor_maps_in:
-        if tm.interpretation == Interpretation.LANGUAGE:
-            language_map = tm
-            if PARTNERS_READ_TEXT in tm.name:
-                index_map = PARTNERS_IDX_2_CHAR
-                char_map = PARTNERS_CHAR_2_IDX
-            else:
-                index_map = ECG_IDX_2_CHAR
-                char_map = ECG_CHAR_2_IDX
-        elif tm.interpretation == Interpretation.EMBEDDING:
-            embed_map = tm
-
-    try:
-        embed_map
-    except NameError:
-        raise ValueError(f'Sampling from a character level model requires an embedding tmap.')
-
-    window_size = test_batch[language_map.input_name()].shape[1]
-    alphabet_size = test_batch[language_map.input_name()].shape[2]
-    for i in range(test_batch[embed_map.input_name()].shape[0]):
-        count = 0
-        sentence = ''
-        next_char = ''
-        embed_in = test_batch[embed_map.input_name()][i:i+1, :]
-        burn_in = np.zeros((1, window_size, alphabet_size), dtype=np.float32)
-        window_size = burn_in.shape[1]
-        with h5py.File(test_paths[i], 'r') as hd5:
-            logging.info(f"\n")
-            if 'read_' in language_map.name:
-                caption = decompress_data(data_compressed=hd5[tm.name][()], dtype=hd5[tm.name].attrs['dtype'])
-            else:
-                caption = str(tm.hd5_first_dataset_in_group(hd5, tm.hd5_key_guess())[()]).strip()
-            logging.info(f"Real text: {caption}")
-        while next_char != '!' and count < 400:
-            cur_test = {embed_map.input_name(): embed_in, language_map.input_name(): burn_in}
-            y_pred = char_model.predict(cur_test)
-            next_char = index_map[_sample_with_heat(y_pred[0, :], 0.7)]
-            sentence += next_char
-            burn_in = np.zeros((1,) + test_batch[language_map.input_name()].shape[1:], dtype=np.float32)
-            for j, c in enumerate(reversed(sentence)):
-                if j == window_size:
-                    break
-                burn_in[0, window_size-j-1, char_map[c]] = 1.0
-            count += 1
-        logging.info(f"Model text:{sentence}")
+        logging.info(f'Model:{sentence}')
 
 
 def tensors_to_label_dictionary(
@@ -435,7 +390,7 @@ def infer_with_pixels(args):
     inference_tsv = os.path.join(args.output_folder, args.id, 'pixel_inference_' + args.id + '.tsv')
     tensor_paths = [args.tensors + tp for tp in sorted(os.listdir(args.tensors)) if os.path.splitext(tp)[-1].lower() == TENSOR_EXT]
     # hard code batch size to 1 so we can iterate over file names and generated tensors together in the tensor_paths for loop
-    model = make_multimodal_multitask_model(**args.__dict__)
+    model = legacy_multimodal_multitask_model(**args.__dict__)
     generate_test = TensorGenerator(
         1, args.tensor_maps_in, args.tensor_maps_out, tensor_paths, num_workers=args.num_workers,
         cache_size=args.cache_size, keep_paths=True, mixup_alpha=args.mixup_alpha,
@@ -778,7 +733,7 @@ class ExploreParallelWrapper():
                         dict_of_tensor_dicts[0][f'error_type_{tm.name}'] = type(e).__name__
 
                 for i in dict_of_tensor_dicts:
-                    dict_of_tensor_dicts[i]['fpath'] = os.path.basename(path).split('.')[0]
+                    dict_of_tensor_dicts[i]['sample_id'] = os.path.basename(path).split('.')[0]
                     dict_of_tensor_dicts[i]['generator'] = gen_name
 
                 # write tdicts to disk
@@ -819,7 +774,7 @@ def _tensors_to_df(args):
     ExploreParallelWrapper(tmaps, paths, args.num_workers, args.output_folder, args.id).run()
 
     # get columns that should have dtype 'string' instead of dtype 'O'
-    str_cols = ['fpath', 'generator']
+    str_cols = ['sample_id', 'generator']
     for tm in tmaps:
         if tm.interpretation == Interpretation.LANGUAGE:
             str_cols.extend([f'{tm.name} {cm}' for cm in tm.channel_map] if tm.channel_map else [tm.name])
@@ -835,7 +790,8 @@ def _tensors_to_df(args):
             fpath = os.path.join(base, name)
             _df = pd.read_csv(fpath, dtype=str_cols)
             logging.debug(f'Loaded {fpath} into memory')
-            df = df.append(_df, ignore_index=True)
+            #df = df.append(_df, ignore_index=True)
+            df = pd.concat([df, _df], ignore_index=True)
             logging.debug(f'Appended {fpath} to overall dataframe')
             temp_files.append(fpath)
 
@@ -894,12 +850,15 @@ def explore(args):
         df = df[cols]
 
     if tsv_style_is_genetics:
-        fid = df['fpath'].str.split('/').str[-1].str.split('.').str[0]
+        fid = df['sample_id'].str.split('/').str[-1].str.split('.').str[0]
         df.insert(0, 'FID', fid)
         df.insert(1, 'IID', fid)
     # Save dataframe to CSV
     fpath = os.path.join(args.output_folder, args.id, f"tensors_all_union.{out_ext}")
     df.to_csv(fpath, index=False, sep=out_sep)
+    df[df.generator=='train'][['sample_id']].to_csv(os.path.join(args.output_folder, args.id, f"train.{out_ext}"), index=False, sep=out_sep)
+    df[df.generator=='test'][['sample_id']].to_csv(os.path.join(args.output_folder, args.id, f"test.{out_ext}"), index=False, sep=out_sep)
+    df[df.generator=='valid'][['sample_id']].to_csv(os.path.join(args.output_folder, args.id, f"valid.{out_ext}"), index=False, sep=out_sep)
     fpath = os.path.join(args.output_folder, args.id, f"tensors_all_intersect.{out_ext}")
     df.dropna().to_csv(fpath, index=False, sep=out_sep)
     logging.info(f"Saved dataframe of tensors (union and intersect) to {fpath}")
@@ -1084,293 +1043,18 @@ def explore(args):
                 logging.info(f"Saved {name} histogram plot at: {figure_path}")
 
 
-
-def cross_reference(args):
-    """Cross reference a source cohort with a reference cohort."""
-    cohort_counts = OrderedDict()
-
-    src_path = args.tensors_source
-    src_name = args.tensors_name
-    src_join = args.join_tensors
-    src_time = args.time_tensor
-    ref_path = args.reference_tensors
-    ref_name = args.reference_name
-    ref_join = args.reference_join_tensors
-    ref_start = args.reference_start_time_tensor
-    ref_end = args.reference_end_time_tensor
-    ref_labels = args.reference_labels
-    number_in_window = args.number_per_window
-    order_in_window = args.order_in_window
-    window_names = args.window_name
-    match_exact_window = order_in_window is not None
-    match_min_window = not match_exact_window
-    match_any_window = args.match_any_window
-    match_every_window = not match_any_window
-
-    # parse options
-    src_cols = list(src_join)
-    ref_cols = list(ref_join)
-    if ref_labels is not None:
-        ref_cols.extend(ref_labels)
-
-    def _cols_from_time_windows(time_windows):
-        return {time_point[0] for time_window in time_windows for time_point in time_window}
-
-    use_time = not any(arg is None for arg in [src_time, ref_start, ref_end])
-    if use_time:
-        if len(ref_start) != len(ref_end):
-            raise ValueError(f"Invalid time windows, got {len(ref_start)} starts and {len(ref_end)} ends")
-
-        if order_in_window is None:
-            # if not matching exactly N in time window, order_in_window is None
-            # make array of blanks so zip doesnt break later
-            order_in_window = [''] * len(ref_start)
-        elif len(order_in_window) != len(ref_start):
-            raise ValueError(f"Ambiguous time selection in time windows, got {len(order_in_window)} order_in_window for {len(ref_start)} windows")
-
-        if window_names is None:
-            window_names = [str(i) for i in range(len(ref_start))]
-        elif len(window_names) != len(ref_start):
-            raise ValueError(f"Ambiguous time window names, got {len(window_names)} names for {len(ref_start)} windows")
-
-        # get time columns and ensure time windows are defined
-        src_cols.append(src_time)
-
-        # ref start and end are lists of lists, defining time windows
-        time_windows = list(zip(ref_start, ref_end))
-
-        # each time window is defined by a tuples of two lists,
-        # where the first list of each tuple defines the start point of the time window
-        # and the second list of each tuple defines the end point of the time window
-        for start, end in time_windows:
-            # each start/end point list is two elements,
-            # where the first element in the list is the name of the time tensor
-            # and the second element is the offset to the value of the time tensor
-
-            # add day offset of 0 for time points without explicit offset
-            [time_point.append(0) for time_point in [start, end] if len(time_point) == 1]
-
-            # parse day offset as int
-            start[1] = int(start[1])
-            end[1] = int(end[1])
-
-        # add unique column names to ref_cols
-        ref_cols.extend(_cols_from_time_windows(time_windows))
-
-    # load data into dataframes
-    def _load_data(name, path, cols):
-        if os.path.isdir(path):
-            logging.debug(f'Assuming {name} is directory of hd5 at {path}')
-            from ml4h.arguments import tensormap_lookup
-            # TODO: check if this works!
-            args.tensor_maps_in = [tensormap_lookup(it) for it in cols]
-            df = _tensors_to_df(args)[cols]
-        else:
-            logging.debug(f'Assuming {name} is a csv at {path}')
-            df = pd.read_csv(path, usecols=cols, low_memory=False)
-        return df
-
-    src_df = _load_data(src_name, src_path, src_cols)
-    logging.info(f'Loaded {src_name} into dataframe')
-    ref_df = _load_data(ref_name, ref_path, ref_cols)
-    logging.info(f'Loaded {ref_name} into dataframe')
-
-    # cleanup time col
-    if use_time:
-        src_df[src_time] = pd.to_datetime(src_df[src_time], errors='coerce', infer_datetime_format=True)
-        src_df.dropna(subset=[src_time], inplace=True)
-
-        for ref_time in _cols_from_time_windows(time_windows):
-            ref_df[ref_time] = pd.to_datetime(ref_df[ref_time], errors='coerce', infer_datetime_format=True)
-        ref_df.dropna(subset=_cols_from_time_windows(time_windows), inplace=True)
-
-        def _add_offset_time(ref_time):
-            offset = ref_time[1]
-            ref_time = ref_time[0]
-            if offset == 0:
-                return ref_time
-            ref_time_col = f'{ref_time}_{offset:+}_days'
-            if ref_time_col not in ref_df:
-                ref_df[ref_time_col] = ref_df[ref_time].apply(lambda x: x + datetime.timedelta(days=offset))
-                ref_cols.append(ref_time_col)
-            return ref_time_col
-
-        # convert time windows to tuples of cleaned and parsed column names
-        time_windows = [(_add_offset_time(start), _add_offset_time(end)) for start, end in time_windows]
-    logging.info('Cleaned data columns and removed rows that could not be parsed')
-
-    # drop duplicates based on cols
-    src_df.drop_duplicates(subset=src_cols, inplace=True)
-    ref_df.drop_duplicates(subset=ref_cols, inplace=True)
-    logging.info('Removed duplicates from dataframes, based on join, time, and label')
-
-    cohort_counts[f'{src_name} (total)'] = len(src_df)
-    cohort_counts[f'{src_name} (unique {" + ".join(src_join)})'] = len(src_df.drop_duplicates(subset=src_join))
-    cohort_counts[f'{ref_name} (total)'] = len(ref_df)
-    cohort_counts[f'{ref_name} (unique {" + ".join(ref_join)})'] = len(ref_df.drop_duplicates(subset=ref_join))
-
-    # merging on join columns duplicates rows in source if there are duplicate join values in both source and reference
-    # this is fine, each row in reference needs all associated rows in source
-    cross_df = src_df.merge(ref_df, how='inner', left_on=src_join, right_on=ref_join).sort_values(src_cols)
-    logging.info('Cross referenced based on join tensors')
-
-    cohort_counts[f'{src_name} in {ref_name} (unique {" + ".join(src_cols)})'] = len(cross_df.drop_duplicates(subset=src_cols))
-    cohort_counts[f'{src_name} in {ref_name} (unique {" + ".join(src_join)})'] = len(cross_df.drop_duplicates(subset=src_join))
-    cohort_counts[f'{ref_name} in {src_name} (unique joins + times + labels)'] = len(cross_df.drop_duplicates(subset=ref_cols))
-    cohort_counts[f'{ref_name} in {src_name} (unique {" + ".join(ref_join)})'] = len(cross_df.drop_duplicates(subset=ref_join))
-
-    # dump results and report label distribution
-    def _report_cross_reference(df, title):
-        title = title.replace(' ', '_')
-        if ref_labels is not None:
-            series = df[ref_labels].astype(str).apply(lambda x: '<>'.join(x), axis=1, raw=True)
-            label_values, counts = np.unique(series, return_counts=True)
-            label_values = np.array(list(map(lambda val: val.split('<>'), label_values)))
-            label_values = np.append(label_values, [['Total']*len(ref_labels)], axis=0)
-            total = sum(counts)
-            counts = np.append(counts, [total])
-            fracs = list(map(lambda f: f'{f:0.5f}', counts / total))
-
-            res = pd.DataFrame(data=label_values, columns=ref_labels)
-            res['count'] = counts
-            res['fraction total'] = fracs
-
-            # save label counts to csv
-            fpath = os.path.join(args.output_folder, args.id, f'label_counts_{title}.csv')
-            res.to_csv(fpath, index=False)
-            logging.info(f'Saved distribution of labels in cross reference to {fpath}')
-
-        # save cross reference to csv
-        fpath = os.path.join(args.output_folder, args.id, f'list_{title}.csv')
-        df.set_index(src_join, drop=True).to_csv(fpath)
-        logging.info(f'Saved cross reference to {fpath}')
-
-    if use_time:
-        # count rows across time windows
-        def _count_time_windows(dfs, title, exact_or_min):
-            if type(dfs) is list:
-                # Number of pre-op (surgdt -180 days; surgdt) ECG from patients with 1+ ECG in all windows
-                # Number of distinct pre-op (surgdt -180 days; surgdt) ECG from patients with 1+ ECG in all windows
-                # Number of distinct pre-op (surgdt -180 days; surgdt) partners_ecg_patientid_clean from patients with 1+ ECG in all windows
-
-                # Number of newest pre-op (surgdt -180 days; surgdt) ECG from patients with 1 ECG in all windows
-                # Number of distinct newest pre-op (surgdt -180 days; surgdt) ECG from patients with 1 ECG in all windows
-                # Number of distinct newest pre-op (surgdt -180 days; surgdt) partners_ecg_patientid_clean from patients with 1 ECG in all windows
-                for df, window_name, order, (start, end) in zip(dfs, window_names, order_in_window, time_windows):
-                    order = f'{order} ' if exact_or_min == 'exactly' else ''
-                    start = start.replace('_', ' ')
-                    end = end.replace('_', ' ')
-                    cohort_counts[f'Number of {order}{window_name} ({start}; {end}) {src_name} from patients with {title}'] = len(df)
-                    cohort_counts[f'Number of distinct {order}{window_name} ({start}; {end}) {src_name} from patients with {title}'] = len(df.drop_duplicates(subset=src_cols))
-                    cohort_counts[f'Number of distinct {order}{window_name} ({start}; {end}) {" + ".join(src_join)} from patients with {title}'] = len(df.drop_duplicates(subset=src_join))
-            else:
-                # Number of ECGs from patients with 1+ ECG in all windows
-                # Number of distinct ECGs from patients with 1+ ECG in all windows
-                # Number of distinct partners_ecg_patientid_clean from patients with 1+ ECG in all windows
-                df = dfs
-                cohort_counts[f'Number of {src_name} from patients with {title}'] = len(df)
-                cohort_counts[f'Number of distinct {src_name} from patients with {title}'] = len(df.drop_duplicates(subset=src_cols))
-                cohort_counts[f'Number of distinct {" + ".join(src_join)} from patients with {title}'] = len(df.drop_duplicates(subset=src_join))
-
-        # aggregate all time windows back into one dataframe with indicator for time window index
-        def _aggregate_time_windows(time_window_dfs, window_names):
-            for df, window_name in zip(time_window_dfs, window_names):
-                if 'time_window' not in df:
-                    df['time_window'] = window_name
-            aggregated_df = pd.concat(time_window_dfs, ignore_index=True).sort_values(by=src_cols + ['time_window'], ignore_index=True)
-            return aggregated_df
-
-        # get only occurrences for join_tensors that appear in every time window
-        def _intersect_time_windows(time_window_dfs):
-            # find the intersection of join_tensors that appear in all time_window_dfs
-            join_tensor_intersect = reduce(lambda a, b: a.merge(b), [pd.DataFrame(df[src_join].drop_duplicates()) for df in time_window_dfs])
-
-            # filter time_window_dfs to only the rows that have join_tensors across all time windows
-            time_window_dfs_intersect = [df.merge(join_tensor_intersect) for df in time_window_dfs]
-            return time_window_dfs_intersect
-
-        # 1. get data with at least N (default 1) occurrences in all time windows
-        # 2. within each time window, get only data for join_tensors that have N rows in the time window
-        # 3. across all time windows, get only data for join_tensors that have data in all time windows
-
-        # get df for each time window
-        dfs_min_in_any_time_window = [
-            cross_df[(cross_df[start] < cross_df[src_time]) & (cross_df[src_time] < cross_df[end])]
-            for start, end in time_windows
-        ]
-
-        # get at least N occurrences in any time window
-        dfs_min_in_any_time_window = [df.groupby(src_join+[start, end]).filter(lambda g: len(g) >= number_in_window) for df, (start, end) in zip(dfs_min_in_any_time_window, time_windows)]
-        if match_min_window and match_any_window:
-            min_in_any_time_window = _aggregate_time_windows(dfs_min_in_any_time_window, window_names)
-            logging.info(f"Cross referenced so unique event occurs {number_in_window}+ times in any time window")
-            title = f'{number_in_window}+ in any window'
-            _report_cross_reference(min_in_any_time_window, title)
-            _count_time_windows(dfs_min_in_any_time_window, title, 'at least')
-            if len(dfs_min_in_any_time_window) > 1:
-                _count_time_windows(min_in_any_time_window, title, 'at least')
-
-        # get at least N occurrences in every time window
-        if match_min_window and match_every_window:
-            dfs_min_in_every_time_window = _intersect_time_windows(dfs_min_in_any_time_window)
-            min_in_every_time_window = _aggregate_time_windows(dfs_min_in_every_time_window, window_names)
-            logging.info(f"Cross referenced so unique event occurs {number_in_window}+ times in all time windows")
-            title = f'{number_in_window}+ in all windows'
-            _report_cross_reference(min_in_every_time_window, title)
-            _count_time_windows(dfs_min_in_every_time_window, title, 'at least')
-            if len(dfs_min_in_every_time_window) > 1:
-                _count_time_windows(min_in_every_time_window, title, 'at least')
-
-        # get exactly N occurrences, select based on ordering
-        def _get_occurrences(df, order, start, end):
-            if order == 'newest':
-                df = df.groupby(src_join+[start, end]).tail(number_in_window)
-            elif order == 'oldest':
-                df = df.groupby(src_join+[start, end]).head(number_in_window)
-            elif order == 'random':
-                df = df.groupby(src_join+[start, end]).apply(lambda g: g.sample(number_in_window))
-            else:
-                raise NotImplementedError(f"Ordering for which rows to use in time window unknown: '{order}'")
-            return df.reset_index(drop=True)
-
-        # get exactly N occurrences in any time window
-        if match_exact_window:
-            dfs_exact_in_any_time_window = [_get_occurrences(df, order, start, end) for df, order, (start, end) in zip(dfs_min_in_any_time_window, order_in_window, time_windows)]
-        if match_exact_window and match_any_window:
-            exact_in_any_time_window = _aggregate_time_windows(dfs_exact_in_any_time_window, window_names)
-            logging.info(f"Cross referenced so unique event occurs exactly {number_in_window} times in any time window")
-            title = f'{number_in_window} in any window'
-            _report_cross_reference(exact_in_any_time_window, title)
-            _count_time_windows(dfs_exact_in_any_time_window, title, 'exactly')
-            if len(dfs_exact_in_any_time_window) > 1:
-                _count_time_windows(exact_in_any_time_window, title, 'exactly')
-
-        # get exactly N occurrences in every time window
-        if match_exact_window and match_every_window:
-            dfs_exact_in_every_time_window = _intersect_time_windows(dfs_exact_in_any_time_window)
-            exact_in_every_time_window = _aggregate_time_windows(dfs_exact_in_every_time_window, window_names)
-            logging.info(f"Cross referenced so unique event occurs exactly {number_in_window} times in all time windows")
-            title = f'{number_in_window} in all windows'
-            _report_cross_reference(exact_in_every_time_window, title)
-            _count_time_windows(dfs_exact_in_every_time_window, title, 'exactly')
-            if len(dfs_exact_in_every_time_window) > 1:
-                _count_time_windows(exact_in_every_time_window, title, 'exactly')
-    else:
-        _report_cross_reference(cross_df, f'all {src_name} in {ref_name}')
-
-    # report counts
-    fpath = os.path.join(args.output_folder, args.id, 'summary_cohort_counts.csv')
-    pd.DataFrame.from_dict(cohort_counts, orient='index', columns=['count']).rename_axis('description').to_csv(fpath)
-    logging.info(f'Saved cohort counts to {fpath}')
-
-
 def latent_space_dataframe(infer_hidden_tsv, explore_csv):
     df = pd.read_csv(explore_csv)
-    df['fpath'] = pd.to_numeric(df['fpath'], errors='coerce')
-    df2 = pd.read_csv(infer_hidden_tsv, sep='\t')
+    if 'sample_id' in df.columns:
+        id_col = 'sample_id'
+    elif 'fpath' in df.columns:
+        id_col = 'fpath'
+    else:
+        raise ValueError(f'Could not find a sample ID column in explore CSV:{explore_csv}')
+    df['sample_id'] = pd.to_numeric(df[id_col], errors='coerce')
+    df2 = pd.read_csv(infer_hidden_tsv, sep='\t', engine='python')
     df2['sample_id'] = pd.to_numeric(df2['sample_id'], errors='coerce')
-    latent_df = pd.merge(df, df2, left_on='fpath', right_on='sample_id', how='inner')
-    latent_df.info()
+    latent_df = pd.merge(df, df2, on='sample_id', how='inner')
     return latent_df
 
 
